﻿// Disclaimer:
// This software is provided AS IS for educational purposes only,
// with no warranties, and confers no rights. Error checking is 
// elided for clarity. Carry out sufficient tests prior to using
// it in a productive environment.
// CASIO Europe GmbH accepts no responsibility for loss or damage
// arising from its use, including damage from virus.
//
// Please, note: This configuration information may be changed
// without any further notice.
// Casio Europe GmbH does not guarantee future compatibility. By 
// using this software you agree to these terms and conditions.
//
// Version 1.01 - January 20, 2011
// Supported Devices: DT-X7, DT-X8, DT-X11, DT-X30, IT-600, 
// IT-800, and IT-3100
//
// Remarks: You need latest versions of CASIO runtime libraries on
// your mobile device. If they are not already installed, you can 
// find them as part of the SDK.
//////////////////////////////////////////////////////////////////////

using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Reflection;
using System.Globalization;
using System.Diagnostics;

namespace BTSerial
{
    internal abstract class LogFileBase : ILogFile
    {
        private int maxLines;
        private string fullyQualifiedName;

        private const string LogFileExtension = ".log";

        protected LogFileBase(string name, int maxLines)
        {
            this.maxLines = maxLines;
            string codeBase = Assembly.GetExecutingAssembly().GetName().CodeBase;
            this.fullyQualifiedName = Path.Combine(
                Path.GetDirectoryName(codeBase),
                Path.GetFileNameWithoutExtension(codeBase) + name + LogFileExtension);
        }

        public void Append(string theEntry)
        {
            Debug.WriteLine(theEntry);

            DateTime dtNow = DateTime.Now;
            string line = null;
            string[] lines = new string[this.maxLines];
            int nLines = default(int);

            if (File.Exists(this.fullyQualifiedName) == true)
            {
                using (StreamReader sr = new StreamReader(this.fullyQualifiedName))
                {
                    for (int i = 0; i < this.maxLines; i++)
                    {
                        line = sr.ReadLine();
                        if (line != null)
                        {
                            lines[i] = line;
                            nLines++;
                        }
                        else
                        {
                            break;
                        }
                    }
                }
            }

            using (StreamWriter sw = new StreamWriter(this.fullyQualifiedName))
            {
                for (int i = (nLines >= this.maxLines) ? 1 : 0; i < this.maxLines; i++)
                {
                    if (lines[i] != null)
                    {
                        sw.WriteLine(lines[i]);
                    }
                    else
                    {
                        break;
                    }
                }

                sw.WriteLine(string.Format(
                    CultureInfo.InvariantCulture, 
                    "{0} {1}: {2}", 
                    dtNow.ToShortDateString(), 
                    dtNow.ToShortTimeString(), 
                    theEntry));
            }
        }
    }
}
